import { App } from 'vue-demi';
import { InjectionKey } from 'vue-demi';

export declare type BindingObject = {
    [key: string]: never;
};

export declare const CheckboxField: ({ value, ...rest }: Partial<CheckboxInput>) => CheckboxInput;

export declare type CheckboxInput = InputBase & {
    type: FieldTypes.CHECKBOX;
    value: boolean;
};

export declare const ColorField: ({ value, ...rest }: Partial<ColorInput>) => ColorInput;

export declare type ColorInput = InputBase & {
    type: FieldTypes.COLOR;
    value: string;
};

export declare type ControlValue = string | number | boolean;

export declare function createDynamicForms(options?: DynamicFormsOptions): DynamicFormsPlugin;

export declare const CustomField: ({ value, ...rest }: Partial<CustomInput>) => CustomInput;

export declare type CustomInput = InputBase & {
    type: FieldTypes.CUSTOM;
    value: boolean | string | number;
};

export declare interface DynamicForm {
    id: string;
    fields: FormFields;
    fieldOrder?: string[];
    options?: FormOptions;
}

export declare interface DynamicFormsOptions {
    autoValidate?: boolean;
    form?: FormOptions;
}

export declare interface DynamicFormsPlugin {
    options?: DynamicFormsOptions;
    install(app: App): void;
}

export declare const dynamicFormsSymbol: InjectionKey<DynamicFormsPlugin>;

export declare const email: (value: string) => ValidationErrors;

export declare const EmailField: ({ value, ...rest }: Partial<EmailInput>) => EmailInput;

export declare type EmailInput = InputBase & {
    type: FieldTypes.EMAIL;
    value: string;
};

export declare interface ErrorMessage {
    text: string;
    value: boolean | null;
}

export declare const FieldBase: ({ validations, label, ariaLabel, ariaLabelledBy, customClass, customStyles, disabled, placeholder, autocomplete, readonly, validationTrigger, }: InputBase) => InputBase;

export declare const FieldControl: ({ name, type, ...rest }: Partial<FormControl<any>>) => FormControl<any>;

export declare const enum FieldTypes {
    TEXT = "text",
    TEXTAREA = "textarea",
    SELECT = "select",
    NUMBER = "number",
    EMAIL = "email",
    URL = "url",
    PASSWORD = "password",
    CHECKBOX = "checkbox",
    RADIO = "radio",
    CUSTOM = "custom-field",
    COLOR = "color"
}

export declare type FormChanges = {
    [key: string]: string | number | null | undefined;
};

export declare type FormControl<T extends InputType> = T & {
    valid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
};

export declare type FormFields = {
    [key: string]: InputType;
};

export declare interface FormOptions {
    customClass?: string | string[] | BindingObject | BindingObject[] | unknown;
    customStyles?: string | string[] | BindingObject | BindingObject[] | unknown;
    method?: string;
    netlify?: boolean;
    netlifyHoneypot?: string;
    autocomplete?: boolean;
}

export declare interface FormValidator {
    validator: ValidatorFn;
    text: string;
    type?: string;
}

export declare interface InputBase {
    name?: string;
    label?: string;
    ariaLabel?: string;
    ariaLabelledBy?: string;
    disabled?: boolean;
    customClass?: string | string[] | BindingObject | BindingObject[] | unknown;
    customStyles?: string | string[] | BindingObject | BindingObject[] | unknown;
    placeholder?: string;
    autocomplete?: string;
    readonly?: boolean;
    validations?: FormValidator[];
    validationTrigger?: ValidationTrigger;
}

declare interface InputEvent_2 {
    name: string;
    value?: unknown;
}
export { InputEvent_2 as InputEvent }

export declare type InputType = TextInput | NumberInput | SelectInput | TextAreaInput | CheckboxInput | RadioInput | EmailInput | PasswordInput | RadioInput | ColorInput | UrlInput | CustomInput;

export declare const isEmptyInputValue: (value: ControlValue) => boolean;

export declare const max: (max: number) => (value: number) => ValidationErrors;

export declare const maxLength: (maxLength: number) => (value: string) => ValidationErrors;

export declare const min: (min: number) => (value: number) => ValidationErrors;

export declare const minLength: (minLength: number) => (value: string) => ValidationErrors;

export declare type MustHave<T, K extends keyof T> = Partial<T> & Pick<T, K>;

export declare const NumberField: ({ value, min, max, step, ...rest }: Partial<NumberInput>) => NumberInput;

export declare type NumberInput = InputBase & {
    type: FieldTypes.NUMBER;
    value: number;
    min: number;
    max: number;
    step: number;
};

export declare const PasswordField: ({ value, ...rest }: Partial<PasswordInput>) => PasswordInput;

export declare type PasswordInput = InputBase & {
    type: FieldTypes.PASSWORD;
    value: string;
};

export declare const pattern: (pattern: string) => ValidatorFn;

export declare const RadioField: ({ options, value, ...rest }: Partial<RadioInput>) => RadioInput;

export declare type RadioInput = InputBase & {
    type: FieldTypes.RADIO;
    value: string;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};

export declare const required: (value: ControlValue) => ValidationErrors;

export declare const SelectField: ({ options, value, optionValue, optionLabel, ...rest }: Partial<SelectInput>) => SelectInput;

export declare type SelectInput<T = boolean | string> = InputBase & {
    type: FieldTypes.SELECT;
    value: T;
    optionValue: string;
    optionLabel: string;
    options?: {
        label: string;
        value: string;
        disabled?: boolean;
    }[];
};

export declare const TextAreaField: ({ value, cols, rows, ...rest }: Partial<TextAreaInput>) => TextAreaInput;

export declare type TextAreaInput = InputBase & {
    type: FieldTypes.TEXTAREA;
    value: string;
    cols?: number;
    rows?: number;
};

export declare const TextField: ({ value, ...rest }: Partial<TextInput>) => TextInput;

export declare type TextInput = InputBase & {
    type: FieldTypes.TEXT;
    value?: string;
};

export declare const url: (value: string) => ValidationErrors;

export declare type UrlInput = InputBase & {
    type: FieldTypes.URL;
    value: string;
};

export declare function useDynamicForms(): DynamicFormsPlugin;

export declare type ValidationErrors = {
    [key: string]: any;
};

export declare type ValidationEvent = InputEvent_2 & {
    errors: ValidationErrors;
    valid: boolean;
};

export declare interface ValidationTrigger {
    type: ValidationTriggerTypes;
    threshold: number;
}

export declare const enum ValidationTriggerTypes {
    BLUR = "blur",
    CHANGE = "change"
}

export declare const Validator: ({ validator, text, }: FormValidator) => FormValidator;

export declare interface ValidatorFn {
    (control: ControlValue | undefined): ValidationErrors | null;
}

export declare const ValidatorTrigger: ({ type, threshold, }: ValidationTrigger) => ValidationTrigger;

export { }
